/*
 * (c) 2010, SWD Embedded Systems Limited, http://www.kpda.ru
 */

#include <string.h>
#include <stdlib.h>
#include <pthread.h>

#define	MAX_BUFF_BYTES	1024
typedef struct buffer buffer_t;

#define BUFF_OVERWRITE		0x01
struct buffer {
	int			head;
	int			tail;
	int			bufsize;
	int			recsize;
	char		*ptr;
	unsigned	mode;
	int			min;
};

extern buffer_t *buff_create( unsigned mode, unsigned size, unsigned rsize );
extern int buff_destroy( buffer_t *bptr );
extern int buff_append( buffer_t *bptr, char *dptr, int n );
extern int buff_delete( buffer_t *bptr, char *dptr, int n );
extern int buff_flush( buffer_t *bptr );
extern int buff_putc( buffer_t *bptr, char c );
extern int buff_getc( buffer_t *bptr );
extern int buff_waiting( buffer_t *bptr );

int buff_destroy( buffer_t *bptr )
{
	free( bptr->ptr );
	free( bptr );
	return( 0 );
}

buffer_t *buff_create( unsigned mode, unsigned size, unsigned rsize )
{
	struct buffer		*cbptr;

	if( cbptr = calloc( sizeof( *cbptr ), 1 ) ) {
		cbptr->recsize = rsize ? rsize : ( rsize = 1 );
		cbptr->bufsize = size ? size * rsize : MAX_BUFF_BYTES - ( MAX_BUFF_BYTES % rsize );
		if( cbptr->ptr = calloc( cbptr->bufsize, 1 ) ) {
			cbptr->mode = mode;
		}
		else {
			free( cbptr );
			cbptr = 0;
		}
	}
	return( cbptr );
}

int buff_append( buffer_t *bptr, char *dptr, int n )
{
	int					t;

	pthread_sleepon_lock( );
	n *= bptr->recsize;
	if( ( t = bptr->tail - bptr->head ) <= 0 ) {
		t += bptr->bufsize;		// calc free size
	}
	if( t <= n ) {
		if( bptr->mode & BUFF_OVERWRITE ) {
			bptr->tail += ( n - t );
			if( bptr->tail >= bptr->bufsize ) {
				bptr->tail = 0;
			}
		}
		else {
			n = t;
		}
	}
	while( n ) {
		t = min( n, bptr->bufsize - bptr->head );
		if( t ) {
			memcpy( bptr->ptr + bptr->head, dptr, t );
			bptr->head += t;
			if( bptr->head >= bptr->bufsize ) {
				bptr->head = 0;
			}
			dptr += t;
			n -= t;
		}
	}
	if( ( t = bptr->head - bptr->tail ) < 0 ) {
		t += bptr->bufsize;
	}
	pthread_sleepon_unlock( );
	return( t );
}

int buff_delete( buffer_t *bptr, char *dptr, int n )
{
	int					t, v = 0;

	pthread_sleepon_lock( );
	n *= bptr->recsize;
	if( ( t = bptr->head - bptr->tail ) < 0 ) {
		t += bptr->bufsize;
	}
	if( n > t ) {
		n = t;
	}
	while( n ) {
		t = min( n, max( 0, bptr->bufsize - bptr->tail ) );
		if( t ) {
			memcpy( dptr, bptr->ptr + bptr->tail, t );
			bptr->tail += t;
			if( bptr->tail >= bptr->bufsize ) {
				bptr->tail = 0;
			}
			dptr += t;
			n -= t;
			v += t;
		}
		else {
			n = 0;
		}
	}
	pthread_sleepon_unlock( );
	return( v );
}

int buff_flush( buffer_t *bptr )
{
	pthread_sleepon_lock( );
	bptr->tail = bptr->head;
	pthread_sleepon_unlock( );
	if( bptr->min < 0 ) {
		bptr->min = 1;
	}
	return( 0 );
}

int buff_putc( buffer_t *bptr, char c )
{
	int					t;

	pthread_sleepon_lock( );
	if( bptr->recsize > 1 ) {
		pthread_sleepon_unlock( );
		return( -1 );
	}
	if( ( t = bptr->tail - bptr->head ) <= 0 ) {
		t += bptr->bufsize;		// calc free size
	}
	if( t <= 1 ) {
		if( bptr->mode & BUFF_OVERWRITE ) {
			if( ++bptr->tail >= bptr->bufsize ) {
				bptr->tail = 0;
			}
			t++;
		}
	}
	if( t ) {
		t--;
		bptr->ptr[bptr->head++] = c;
		if( bptr->head >= bptr->bufsize ) {
			bptr->head = 0;
		}
	}
	pthread_sleepon_unlock( );
	return( bptr->bufsize - t );
}

int buff_getc( buffer_t *bptr )
{
	int					c;

	pthread_sleepon_lock( );
	if( bptr->recsize > 1 || bptr->head == bptr->tail ) {
		pthread_sleepon_unlock( );
		return( -1 );
	}
	c = bptr->ptr[bptr->tail];
	if( ++bptr->tail >= bptr->bufsize ) {
		bptr->tail = 0;
	}
	pthread_sleepon_unlock( );
	return( c );
}

int buff_waiting( buffer_t *bptr )
{
	int					t;

//	pthread_sleepon_lock( );
	if( ( t = bptr->head - bptr->tail ) < 0 ) {
		t += bptr->bufsize;
	}
//	pthread_sleepon_unlock( );
	return( t / bptr->recsize );
}
