/*
 * (c) 2010-2019, SWD Embedded Systems Limited, http://www.kpda.ru
 */

// Module Description:  header for mouse class driver

#include <share.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <termios.h>
#include <gulliver.h>
#include <sys/usbdi.h>
#include <sys/iofunc.h>
#include <sys/resmgr.h>
#include <sys/procmgr.h>
#include <sys/dcmd_chr.h>
#include <sys/dispatch.h>
#include <sys/neutrino.h>
#include <sys/rsrcdbmgr.h>

// private headers
#include <queue.h>

typedef struct buffer buffer_t;

#define BUFF_OVERWRITE		0x01
struct buffer {
	int			head;
	int			tail;
	int			bufsize;
	int			recsize;
	char		*ptr;
	unsigned	mode;
	int			min;
};

//                       MOUSE CLASS DRIVER STRUCTURE
typedef struct _mouse {
	iofunc_attr_t					attr;
	TAILQ_ENTRY( _mouse )			dlink;
	int								resmgr_id;
	_uint32							devnum;
	resmgr_connect_funcs_t			resmgr_connect_funcs;
	resmgr_io_funcs_t				resmgr_io_funcs;
	iofunc_mount_t					io_mount;
	struct termios					tios;
	struct usbd_connection			*connection;
	usbd_device_instance_t			instance;
	struct usbd_device				*device;
	_uint32							status;
	struct usbd_pipe				*ep_cntl;
	struct usbd_pipe				*ep_int;
	uint32_t						ep_int_size;
	struct usbd_urb					*urb;
	_uint8							*ep_int_buf;
	buffer_t						*rbuf;			// ring buffer
} mouse_t;

typedef struct _mouse_ctrl {
	TAILQ_HEAD(,_mouse)				dlist;		// linked list of mice
	_uint32							cflags;
	_uint32							verbose;
	char							*prefix;
	int								coid;
	int								chid;
	thread_pool_t 					*thread_pool;
	dispatch_t						*dispatch;
	thread_pool_attr_t				pool_attr;
} mouse_ctrl_t;

//                          MOUSE HID DEFINITION
#define	USB_SUBCLASS_HID_BOOT	1
#define	USB_PROTOCOL_MOUSE		2

//                    MOUSE BUFFER REPORT DEFINITION
typedef struct _mouse_report {
	_uint8			buttons;
	_uint8			horizontal_position;
	_uint8			vertical_position;
} mouse_report_t;

//                          MOUSE DEFINITION
#define MOUSE_OFF					0
#define MOUSE_ON					1
#define MOUSE_UNBLOCK				2
#define MOUSE_PRESENT				4

#define MOUSE_CONTROL_EP			1
#define MOUSE_INTIN_EP				2

#define MOUSE_BUTTON_1				1
#define MOUSE_BUTTON_2				2
#define MOUSE_BUTTON_3				4
#define MOUSE_BUTTON_LAST_1 		8
#define MOUSE_BUTTON_LAST_2			0x10
#define MOUSE_BUTTON_LAST_3			0x20
#define MOUSE_INCREMENTS			5

#define MOUSE_MOVE					1
#define MOUSE_LEFT_PRESSED			2
#define MOUSE_LEFT_RELEASED			4
#define MOUSE_RIGHT_PRESSED 		8
#define MOUSE_RIGHT_RELEASED		0x10
#define MOUSE_MIDDLE_PRESSED 		0x20
#define MOUSE_MIDDLE_RELEASED		0x40

extern buffer_t *buff_create( unsigned mode, unsigned size, unsigned rsize );
extern int buff_destroy( buffer_t *bptr );
extern int buff_append( buffer_t *bptr, char *dptr, int n );
extern int buff_delete( buffer_t *bptr, char *dptr, int n );
extern int buff_flush( buffer_t *bptr );
extern int buff_putc( buffer_t *bptr, char c );
extern int buff_getc( buffer_t *bptr );
extern int buff_waiting( buffer_t *bptr );
