/*
 * (c) 2010-2019, SWD Embedded Systems Limited, http://www.kpda.ru
 */

// Module Description:  header for printer class driver

#include <share.h>
#include <errno.h>
#include <stdio.h>
#include <atomic.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <termios.h>
#include <gulliver.h>
#include <sys/usbdi.h>
#include <sys/iofunc.h>
#include <sys/resmgr.h>
#include <sys/procmgr.h>
#include <sys/dcmd_chr.h>
#include <sys/dispatch.h>
#include <sys/neutrino.h>
#include <sys/rsrcdbmgr.h>

// private headers
#include <queue.h>

//                       PRINTER CLASS DRIVER STRUCTURE
typedef struct _prn {
	iofunc_attr_t					attr;
	TAILQ_ENTRY( _prn )				dlink;
	int								resmgr_id;
	_uint32							devnum;
	resmgr_connect_funcs_t			resmgr_connect_funcs;
	resmgr_io_funcs_t				resmgr_io_funcs;
	iofunc_mount_t					io_mount;
	struct termios					tios;
	struct usbd_connection			*connection;
	usbd_device_instance_t			instance;
	struct usbd_device				*device;
	_uint32							status;
	struct usbd_pipe				*ep_cntl;
	struct usbd_pipe				*ep_in;
	struct usbd_pipe				*ep_out;
	struct usbd_urb					*urb;
	_uint32							bcdUSB;
	_uint32							protocol;
	_uint32							query_status;
	_uint8							*buffer;
}  prn_t;

typedef struct _prn_ctrl {
	TAILQ_HEAD(,_prn)				dlist;		// linked list of prn's
	_uint32							cflags;
	_uint32							verbose;
	char							*prefix;
	int								coid;
	int								chid;
	thread_pool_t 					*thread_pool;
	dispatch_t						*dispatch;
	thread_pool_attr_t				pool_attr;
} prn_ctrl_t;

//                           PRINTER CLASS DEFINITION
#ifndef USB_CLASS_PRN
#define	USB_CLASS_PRN					7
#endif
#define	USB_SUBCLASS_PRN				1
#define	USB_PROTOCOL_PRN_UNIDIR			1
#define	USB_PROTOCOL_PRN_BIDIR			2
#define	GET_DEVICE_ID					0
#define	GET_PORT_STATUS					1
#define	SOFT_RESET						2

#define PRN_OFF							0x00
#define PRN_ON							0x01
#define PRN_UNBLOCK						0x02
#define PRN_PRESENT						0x04
#define PRN_READING						0x08
#define PRN_WRITING						0x10

#define PRN_CONTROL_EP					1
#define PRN_BULKIN_EP					2
#define PRN_BULKOUT_EP					4

#define	MAX_PRN_BUFFER					512
#define	MAX_PRN_DEVICE_NAME				512
#define PRN_MAX_ENDPOINT				16
#define PRN_BUFFER_SIZE					1536

#define PRN_GET_DEVICE_ID				0
#define PRN_GET_PORT_STATUS				1
#define PRN_SOFT_RESET					2
